function show_sparse_grid_data()

% read sparse grid data
fid = fopen('sparse_grid_data.dat');
data = cell2mat(textscan(fid, '%f\t%f\t%f\t%f\t%f\t%f\n', 'CommentStyle', '#'));
fclose(fid);

% extract sparse grid points: 2D levels/indices and T/B data
L = data(:,1:2);
I = data(:,3:4);
T = data(:,5);
B = data(:,6);

% compute sparse grid points scaled to [0, 1]^2
X = I .* 2.^(-L);
% unscaled points
Theta = 140 * X(:,1) + 10;
Alpha = X(:,2);

% compute matrix of interpolation linear system
N = size(L, 1);
A = nan(N);

for k = 1:N
    A(:,k) = eval_hierarchical_bspline(L(k,:), I(k,:), X);
end

% calculate hierarchical surpluses for T and B
surpluses = A \ [T, B];
surpluses_T = surpluses(:,1);
surpluses_B = surpluses(:,2);

% create plot/evaluation grid
plot_theta = linspace(10, 150, 101);
plot_alpha = linspace(0, 1, 101);
[plot_Theta, plot_Alpha] = meshgrid(plot_theta, plot_alpha);
plot_X = [(plot_Theta(:) - 10) / 140, plot_Alpha(:)];

% evaluate sparse grid spline interpolant
% (note: in the paper, we do not use MATLAB to create the
% sparse grid solution, but the sparse grid toolbox SG++ written in C++)
plot_T = zeros(numel(plot_Theta), 1);
plot_B = zeros(numel(plot_Theta), 1);

for k = 1:N
    bspline_value = eval_hierarchical_bspline(L(k,:), I(k,:), plot_X);
    plot_T = plot_T + surpluses_T(k) * bspline_value;
    plot_B = plot_B + surpluses_B(k) * bspline_value;
end

% reshape data vector to meshgrid-sized matrices
plot_T = reshape(plot_T, size(plot_Theta));
plot_B = reshape(plot_B, size(plot_Theta));

% plot T interpolant and sparse grid points
figure();
hold('on');
surf(plot_Theta, plot_Alpha, plot_T);
plot3(Theta, Alpha, T, 'k.', 'MarkerSize', 12);
xlabel('theta [degrees]');
ylabel('alpha_T');
zlabel('T [N]');
xlim([10 150]);
ylim([0 1]);
title('Sparse grid data/interpolant for T');
grid('on');
view(3);

% plot B interpolant and sparse grid points
figure();
hold('on');
surf(plot_Theta, plot_Alpha, plot_B);
plot3(Theta, Alpha, B, 'k.', 'MarkerSize', 12);
xlabel('theta [degrees]');
ylabel('alpha_B');
zlabel('B [N]');
xlim([10 150]);
ylim([0 1]);
title('Sparse grid data/interpolant for B');
grid('on');
view(3);

end

%% function for evaluating modified hierarchical B-splines via tensor-product
function Y = eval_hierarchical_bspline(L, I, X)

[N, d] = size(X);
Y = ones(N, 1);

for t = 1:d
    % compute tensor product
    Y = Y .* eval_hierarchical_bspline_1d(L(t), I(t), X(:,t));
end

end

%% function for evaluating modified hierarchical 1D B-splines
function y = eval_hierarchical_bspline_1d(l, i, x)

% scaled/translated coordinate (t = 0 corresponds to i * 2^(-l))
t = 2^l * x - i + 2;

if l == 1
    % modification on level 1
    y = ones(size(x));
elseif i == 1
    % modification on higher levels
    y = eval_cardinal_bspline(t) + ...
        2 * eval_hierarchical_bspline_1d(l, 0, x) + ...
        3 * eval_hierarchical_bspline_1d(l, -1, x);
elseif i == 2^l - 1
    % do modification after mirroring
    y = eval_hierarchical_bspline_1d(l, 1, 1 - x);
else
    % unmodified case
    y = eval_cardinal_bspline(t);
end

end

%% function for evaluating the cardinal cubic B-spline
function y = eval_cardinal_bspline(x)

% y(I) will be zero for I = (x <= 0) | (x >= 4)
y = zeros(size(x));

% cubic polynomial sections
I = (x >= 0) & (x < 1);
y(I) = 1/6 * x(I).^3;

I = (x >= 1) & (x < 2);
y(I) = 1/6 * (-3 * x(I).^3 + 12 * x(I).^2 - 12 * x(I) + 4);

I = (x >= 2) & (x < 3);
y(I) = 1/6 * (3 * x(I).^3 - 24 * x(I).^2 + 60 * x(I) - 44);

I = (x >= 3) & (x < 4);
y(I) = 1/6 * (-x(I).^3 + 12 * x(I).^2 - 48 * x(I) + 64);

end
